@@ -2,7 +2,7 @@
 #of this repository contains the full copyright notices and license terms.
 from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.pyson import Not, Equal, Eval, Or, Bool
-from trytond.backend import TableHandler
+from trytond import backend
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
@@ -21,14 +21,14 @@ class Inventory(Workflow, ModelSQL, ModelView):
         'stock.location', 'Location', required=True,
         domain=[('type', '=', 'storage')], states={
             'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('lines'))),
+                Bool(Eval('lines', [0]))),
             },
-        depends=['state', 'lines'])
+        depends=['state'])
     date = fields.Date('Date', required=True, states={
             'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('lines'))),
+                Bool(Eval('lines', [0]))),
             },
-        depends=['state', 'lines'])
+        depends=['state'])
     lost_found = fields.Many2One(
         'stock.location', 'Lost and Found', required=True,
         domain=[('type', '=', 'lost_found')], states=STATES, depends=DEPENDS)
@@ -38,9 +38,9 @@ class Inventory(Workflow, ModelSQL, ModelView):
     company = fields.Many2One('company.company', 'Company', required=True,
         states={
             'readonly': Or(Not(Equal(Eval('state'), 'draft')),
-                Bool(Eval('lines'))),
+                Bool(Eval('lines', [0]))),
             },
-        depends=['state', 'lines'])
+        depends=['state'])
     state = fields.Selection([
         ('draft', 'Draft'),
         ('done', 'Done'),
@@ -54,6 +54,8 @@ class Inventory(Workflow, ModelSQL, ModelView):
         cls._error_messages.update({
                 'delete_cancel': ('Inventory "%s" must be cancelled before '
                     'deletion.'),
+                'unique_line': ('Line "%s" is not unique '
+                    'on Inventory "%s".'),
                 })
         cls._transitions |= set((
                 ('draft', 'done'),
@@ -73,6 +75,7 @@ class Inventory(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         super(Inventory, cls).__register__(module_name)
         cursor = Transaction().cursor
 
@@ -114,14 +117,21 @@ class Inventory(Workflow, ModelSQL, ModelView):
     @Workflow.transition('done')
     def confirm(self, inventories):
         Move = Pool().get('stock.move')
-        move_ids = []
+        moves = []
         for inventory in inventories:
+            keys = set()
             for line in inventory.lines:
-                move_id = line.create_move()
-                if move_id:
-                    move_ids.append(move_id)
-        if move_ids:
-            Move.do(Move.browse(move_ids))
+                key = line.unique_key
+                if key in keys:
+                    self.raise_user_error('unique_line',
+                        (line.rec_name, inventory.rec_name))
+                keys.add(key)
+                move = line.get_move()
+                if move:
+                    moves.append(move)
+        if moves:
+            moves = Move.create([m._save_values for m in moves])
+            Move.do(moves)
 
     @classmethod
     @ModelView.button
@@ -149,7 +159,7 @@ class Inventory(Workflow, ModelSQL, ModelView):
             Line.copy(inventory.lines,
                 default={
                     'inventory': new_inventory.id,
-                    'move': None,
+                    'moves': None,
                     })
             cls.complete_lines([new_inventory])
             new_inventories.append(new_inventory)
@@ -164,6 +174,7 @@ class Inventory(Workflow, ModelSQL, ModelView):
         Line = pool.get('stock.inventory.line')
         Product = pool.get('product.product')
 
+        to_create = []
         for inventory in inventories:
             # Compute product quantities
             with Transaction().set_context(stock_date_end=inventory.date):
@@ -200,7 +211,6 @@ class Inventory(Workflow, ModelSQL, ModelView):
                     Line.write([line], values)
 
             # Create lines if needed
-            to_create = []
             for product_id in product_qty:
                 if (product2type[product_id] != 'goods'
                         or product2consumable[product_id]):
@@ -209,8 +219,8 @@ class Inventory(Workflow, ModelSQL, ModelView):
                 values = Line.create_values4complete(product_id, inventory,
                     quantity, uom_id)
                 to_create.append(values)
-            if to_create:
-                Line.create(to_create)
+        if to_create:
+            Line.create(to_create)
 
 
 class InventoryLine(ModelSQL, ModelView):
@@ -221,8 +231,7 @@ class InventoryLine(ModelSQL, ModelView):
         domain=[
             ('type', '=', 'goods'),
             ('consumable', '=', False),
-            ],
-        on_change=['product'])
+            ])
     uom = fields.Function(fields.Many2One('product.uom', 'UOM'), 'get_uom')
     unit_digits = fields.Function(fields.Integer('Unit Digits'),
             'get_unit_digits')
@@ -231,7 +240,7 @@ class InventoryLine(ModelSQL, ModelView):
             depends=['unit_digits'])
     quantity = fields.Float('Quantity', required=True,
         digits=(16, Eval('unit_digits', 2)), depends=['unit_digits'])
-    move = fields.Many2One('stock.move', 'Move', readonly=True)
+    moves = fields.One2Many('stock.move', 'origin', 'Moves', readonly=True)
     inventory = fields.Many2One('stock.inventory', 'Inventory', required=True,
             ondelete='CASCADE')
 
@@ -241,11 +250,35 @@ class InventoryLine(ModelSQL, ModelView):
         cls._sql_constraints += [
             ('check_line_qty_pos',
                 'CHECK(quantity >= 0.0)', 'Line quantity must be positive.'),
-            ('inventory_product_uniq', 'UNIQUE(inventory, product)',
-                'Product must be unique by inventory.'),
             ]
         cls._order.insert(0, ('product', 'ASC'))
 
+    @classmethod
+    def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
+        cursor = Transaction().cursor
+        pool = Pool()
+        Move = pool.get('stock.move')
+        sql_table = cls.__table__()
+        move_table = Move.__table__()
+
+        super(InventoryLine, cls).__register__(module_name)
+
+        table = TableHandler(cursor, cls, module_name)
+        # Migration from 2.8: Remove constraint inventory_product_uniq
+        table.drop_constraint('inventory_product_uniq')
+
+        # Migration from 3.0: use Move origin
+        if table.column_exist('move'):
+            cursor.execute(*sql_table.select(sql_table.id, sql_table.move,
+                    where=sql_table.move != None))
+            for line_id, move_id in cursor.fetchall():
+                cursor.execute(*move_table.update(
+                        columns=[move_table.origin],
+                        values=['%s,%s' % (cls.__name__, line_id)],
+                        where=move_table.id == move_id))
+            table.drop_column('move')
+
     @staticmethod
     def default_unit_digits():
         return 2
@@ -254,6 +287,7 @@ class InventoryLine(ModelSQL, ModelView):
     def default_expected_quantity():
         return 0.
 
+    @fields.depends('product')
     def on_change_product(self):
         change = {}
         change['unit_digits'] = 2
@@ -263,24 +297,29 @@ class InventoryLine(ModelSQL, ModelView):
             change['unit_digits'] = self.product.default_uom.digits
         return change
 
+    def get_rec_name(self, name):
+        return self.product.rec_name
+
     def get_uom(self, name):
         return self.product.default_uom.id
 
     def get_unit_digits(self, name):
         return self.product.default_uom.digits
 
+    @property
+    def unique_key(self):
+        return (self.product,)
+
     @classmethod
     def cancel_move(cls, lines):
         Move = Pool().get('stock.move')
-        Move.cancel([l.move for l in lines if l.move])
-        Move.delete([l.move for l in lines if l.move])
-        cls.write([l for l in lines if l.move], {
-            'move': None,
-            })
+        moves = [m for l in lines for m in l.moves if l.moves]
+        Move.cancel(moves)
+        Move.delete(moves)
 
-    def create_move(self):
+    def get_move(self):
         '''
-        Create move for an inventory line and return id
+        Return Move instance for the inventory line
         '''
         pool = Pool()
         Move = pool.get('stock.move')
@@ -291,24 +330,22 @@ class InventoryLine(ModelSQL, ModelView):
             self.uom)
         if delta_qty == 0.0:
             return
-        from_location = self.inventory.location.id
-        to_location = self.inventory.lost_found.id
+        from_location = self.inventory.location
+        to_location = self.inventory.lost_found
         if delta_qty < 0:
             (from_location, to_location, delta_qty) = \
                 (to_location, from_location, -delta_qty)
 
-        move, = Move.create([{
-                    'from_location': from_location,
-                    'to_location': to_location,
-                    'quantity': delta_qty,
-                    'product': self.product.id,
-                    'uom': self.uom.id,
-                    'company': self.inventory.company.id,
-                    'effective_date': self.inventory.date,
-                    }])
-        self.move = move
-        self.save()
-        return move.id
+        return Move(
+            from_location=from_location,
+            to_location=to_location,
+            quantity=delta_qty,
+            product=self.product,
+            uom=self.uom,
+            company=self.inventory.company,
+            effective_date=self.inventory.date,
+            origin=self,
+            )
 
     def update_values4complete(self, quantity, uom_id):
         '''
