@@ -1,14 +1,5 @@
-#!/usr/bin/env python
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-
-import sys
-import os
-DIR = os.path.abspath(os.path.normpath(os.path.join(__file__,
-    '..', '..', '..', '..', '..', 'trytond')))
-if os.path.isdir(DIR):
-    sys.path.insert(0, os.path.dirname(DIR))
-
 import unittest
 import doctest
 import datetime
@@ -17,15 +8,13 @@ from dateutil.relativedelta import relativedelta
 from functools import partial
 import trytond.tests.test_tryton
 from trytond.tests.test_tryton import POOL, DB_NAME, USER, CONTEXT, test_view,\
-    test_depends
-from trytond.backend.sqlite.database import Database as SQLiteDatabase
+    test_depends, doctest_dropdb
 from trytond.transaction import Transaction
+from trytond.exceptions import UserWarning
 
 
 class StockTestCase(unittest.TestCase):
-    '''
-    Test Stock module.
-    '''
+    'Test Stock module'
 
     def setUp(self):
         trytond.tests.test_tryton.install_module('stock')
@@ -41,21 +30,15 @@ class StockTestCase(unittest.TestCase):
         self.cache = POOL.get('stock.period.cache')
 
     def test0005views(self):
-        '''
-        Test views.
-        '''
+        'Test views'
         test_view('stock')
 
     def test0006depends(self):
-        '''
-        Test depends.
-        '''
+        'Test depends'
         test_depends()
 
     def test0010move_internal_quantity(self):
-        '''
-        Test Move.internal_quantity.
-        '''
+        'Test Move.internal_quantity'
         with Transaction().start(DB_NAME, USER, context=CONTEXT):
             category, = self.category.create([{
                         'name': 'Test Move.internal_quantity',
@@ -76,7 +59,9 @@ class StockTestCase(unittest.TestCase):
                         }])
             supplier, = self.location.search([('code', '=', 'SUP')])
             storage, = self.location.search([('code', '=', 'STO')])
-            company, = self.company.search([('rec_name', '=', 'B2CK')])
+            company, = self.company.search([
+                    ('rec_name', '=', 'Dunder Mifflin'),
+                    ])
             currency = company.currency
             self.user.write([self.user(USER)], {
                 'main_company': company.id,
@@ -84,11 +69,12 @@ class StockTestCase(unittest.TestCase):
                 })
 
             tests = [
-                (kg, 10, 10),
-                (g, 100, 0.1),
-                (g, 1, 0),  # rounded
+                (kg, 10, 10, 0),
+                (g, 100, 0.1, 1),
+                (g, 1, 0, 0),  # rounded
+                (kg, 35.23, 35.23, 2),  # check infinite loop
             ]
-            for uom, quantity, internal_quantity in tests:
+            for uom, quantity, internal_quantity, ndigits in tests:
                 move, = self.move.create([{
                             'product': product.id,
                             'uom': uom.id,
@@ -99,19 +85,19 @@ class StockTestCase(unittest.TestCase):
                             'unit_price': Decimal('1'),
                             'currency': currency.id,
                             }])
-                self.assertEqual(move.internal_quantity, internal_quantity)
+                self.assertEqual(round(move.internal_quantity, ndigits),
+                    internal_quantity)
 
-                for uom, quantity, internal_quantity in tests:
+                for uom, quantity, internal_quantity, ndigits in tests:
                     self.move.write([move], {
                         'uom': uom.id,
                         'quantity': quantity,
                         })
-                    self.assertEqual(move.internal_quantity, internal_quantity)
+                    self.assertEqual(round(move.internal_quantity, ndigits),
+                        internal_quantity)
 
     def test0020products_by_location(self):
-        '''
-        Test products_by_location.
-        '''
+        'Test products_by_location'
         with Transaction().start(DB_NAME, USER,
                 context=CONTEXT) as transaction:
             category, = self.category.create([{
@@ -134,7 +120,9 @@ class StockTestCase(unittest.TestCase):
             supplier, = self.location.search([('code', '=', 'SUP')])
             customer, = self.location.search([('code', '=', 'CUS')])
             storage, = self.location.search([('code', '=', 'STO')])
-            company, = self.company.search([('rec_name', '=', 'B2CK')])
+            company, = self.company.search([
+                    ('rec_name', '=', 'Dunder Mifflin'),
+                    ])
             currency = company.currency
             self.user.write([self.user(USER)], {
                 'main_company': company.id,
@@ -210,9 +198,6 @@ class StockTestCase(unittest.TestCase):
 
             products_by_location = partial(self.product.products_by_location,
                     [storage.id], [product.id])
-            products_by_location_zero = partial(
-                    self.product.products_by_location,
-                    [storage.id], [product.id], skip_zero=False)
 
             tests = [
                 ({'stock_date_end': today + relativedelta(days=-6),
@@ -269,16 +254,83 @@ class StockTestCase(unittest.TestCase):
                 }, 6),
             ]
 
+            def tests_product_quantity(context, quantity):
+                with transaction.set_context(locations=[storage.id]):
+                    product_reloaded = self.product(product.id)
+                    if (not context.get('stock_date_end')
+                            or context['stock_date_end'] > today
+                            or context.get('forecast')):
+                        self.assertEqual(product_reloaded.forecast_quantity,
+                            quantity)
+                    else:
+                        self.assertEqual(product_reloaded.quantity, quantity)
+
+            def tests_product_search_quantity(context, quantity):
+                with transaction.set_context(locations=[storage.id]):
+                    if (not context.get('stock_date_end')
+                            or context['stock_date_end'] > today
+                            or context.get('forecast')):
+                        fname = 'forecast_quantity'
+                    else:
+                        fname = 'quantity'
+                    found_products = self.product.search([
+                            (fname, '=', quantity),
+                            ])
+                    self.assertIn(product, found_products)
+
+                    found_products = self.product.search([
+                            (fname, '!=', quantity),
+                            ])
+                    self.assertNotIn(product, found_products)
+
+                    found_products = self.product.search([
+                            (fname, 'in', (quantity, quantity + 1)),
+                            ])
+                    self.assertIn(product, found_products)
+
+                    found_products = self.product.search([
+                            (fname, 'not in', (quantity, quantity + 1)),
+                            ])
+                    self.assertNotIn(product, found_products)
+
+                    found_products = self.product.search([
+                            (fname, '<', quantity),
+                            ])
+                    self.assertNotIn(product, found_products)
+                    found_products = self.product.search([
+                            (fname, '<', quantity + 1),
+                            ])
+                    self.assertIn(product, found_products)
+
+                    found_products = self.product.search([
+                            (fname, '>', quantity),
+                            ])
+                    self.assertNotIn(product, found_products)
+                    found_products = self.product.search([
+                            (fname, '>', quantity - 1),
+                            ])
+                    self.assertIn(product, found_products)
+
+                    found_products = self.product.search([
+                            (fname, '>=', quantity),
+                            ])
+                    self.assertIn(product, found_products)
+
+                    found_products = self.product.search([
+                            (fname, '<=', quantity),
+                            ])
+                    self.assertIn(product, found_products)
+
             def test_products_by_location():
                 for context, quantity in tests:
                     with transaction.set_context(context):
                         if not quantity:
                             self.assertEqual(products_by_location(), {})
-                            self.assertEqual(products_by_location_zero(),
-                                    {(storage.id, product.id): quantity})
                         else:
                             self.assertEqual(products_by_location(),
                                     {(storage.id, product.id): quantity})
+                            tests_product_quantity(context, quantity)
+                            tests_product_search_quantity(context, quantity)
 
             test_products_by_location()
 
@@ -288,7 +340,7 @@ class StockTestCase(unittest.TestCase):
                 today + relativedelta(days=-4),
                 today + relativedelta(days=-3),
                 today + relativedelta(days=-2),
-            ]
+                ]
 
             moves = self.move.create([{
                         'product': product.id,
@@ -315,10 +367,12 @@ class StockTestCase(unittest.TestCase):
                 self.period.close([period])
                 test_products_by_location()
 
-        # Test with_childs
+        # Test with_childs and stock_skip_warehouse
         with Transaction().start(DB_NAME, USER,
                 context=CONTEXT) as transaction:
-            company, = self.company.search([('rec_name', '=', 'B2CK')])
+            company, = self.company.search([
+                    ('rec_name', '=', 'Dunder Mifflin'),
+                    ])
             self.user.write([self.user(USER)], {
                 'main_company': company.id,
                 'company': company.id,
@@ -340,6 +394,7 @@ class StockTestCase(unittest.TestCase):
             lost_found, = self.location.search([('type', '=', 'lost_found')])
             warehouse, = self.location.search([('type', '=', 'warehouse')])
             storage, = self.location.search([('code', '=', 'STO')])
+            input_, = self.location.search([('code', '=', 'IN')])
             storage1, = self.location.create([{
                         'name': 'Storage 1',
                         'type': 'view',
@@ -365,6 +420,15 @@ class StockTestCase(unittest.TestCase):
                         'planned_date': today,
                         'effective_date': today,
                         'company': company.id,
+                        }, {
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 1,
+                        'from_location': input_.id,
+                        'to_location': storage.id,
+                        'planned_date': today,
+                        'effective_date': today,
+                        'company': company.id,
                         }])
             self.move.do(moves)
 
@@ -373,11 +437,20 @@ class StockTestCase(unittest.TestCase):
             self.assertEqual(products_by_location[(warehouse.id, product.id)],
                 1)
 
+            with Transaction().set_context(stock_skip_warehouse=True):
+                products_by_location = self.product.products_by_location(
+                    [warehouse.id], [product.id], with_childs=True)
+                products_by_location_all = self.product.products_by_location(
+                    [warehouse.id], None, with_childs=True)
+            self.assertEqual(products_by_location[(warehouse.id, product.id)],
+                2)
+            self.assertEqual(
+                products_by_location_all[(warehouse.id, product.id)], 2)
+
     def test0030period(self):
-        '''
-        Test period.
-        '''
-        with Transaction().start(DB_NAME, USER, context=CONTEXT):
+        'Test period'
+        with Transaction().start(DB_NAME, USER,
+                context=CONTEXT) as transaction:
             category, = self.category.create([{
                         'name': 'Test period',
                         }])
@@ -397,7 +470,9 @@ class StockTestCase(unittest.TestCase):
             supplier, = self.location.search([('code', '=', 'SUP')])
             customer, = self.location.search([('code', '=', 'CUS')])
             storage, = self.location.search([('code', '=', 'STO')])
-            company, = self.company.search([('rec_name', '=', 'B2CK')])
+            company, = self.company.search([
+                    ('rec_name', '=', 'Dunder Mifflin'),
+                    ])
             currency = company.currency
             self.user.write([self.user(USER)], {
                 'main_company': company.id,
@@ -441,6 +516,18 @@ class StockTestCase(unittest.TestCase):
                         'currency': currency.id,
                         }])
             self.move.do(moves)
+            self.move.create([{
+                        'product': product.id,
+                        'uom': unit.id,
+                        'quantity': 3,
+                        'from_location': supplier.id,
+                        'to_location': storage.id,
+                        'planned_date': None,
+                        'effective_date': None,
+                        'company': company.id,
+                        'unit_price': Decimal('1'),
+                        'currency': currency.id,
+                        }])
 
             tests = [
                 (-5, {
@@ -454,6 +541,29 @@ class StockTestCase(unittest.TestCase):
                 })
             ]
 
+            products_by_location = partial(self.product.products_by_location,
+                [storage.id], [product.id])
+
+            tests_pbl = [
+                ({'stock_date_end': today + relativedelta(days=-6)}, 0),
+                ({'stock_date_end': today + relativedelta(days=-5)}, 10),
+                ({'stock_date_end': today + relativedelta(days=-4)}, 25),
+                ({'stock_date_end': today + relativedelta(days=-3)}, 20),
+                ({'stock_date_end': today + relativedelta(days=-2)}, 20),
+                ({'stock_date_end': today}, 20),
+                ({'stock_date_end': datetime.date.max}, 23),
+                ]
+
+            def test_products_by_location():
+                for context, quantity in tests_pbl:
+                    with transaction.set_context(context):
+                        if not quantity:
+                            self.assertEqual(products_by_location(), {})
+                        else:
+                            self.assertEqual(products_by_location(),
+                                    {(storage.id, product.id): quantity})
+
+            test_products_by_location()
             for days, quantities in tests:
                 period, = self.period.create([{
                             'date': today + relativedelta(days=days),
@@ -469,6 +579,8 @@ class StockTestCase(unittest.TestCase):
                     self.assertEqual(cache.internal_quantity,
                         quantities[cache.location.id])
 
+                test_products_by_location()
+
             # Test check_period_closed
             moves = self.move.create([{
                         'product': product.id,
@@ -510,18 +622,42 @@ class StockTestCase(unittest.TestCase):
                         }])
             self.assertRaises(Exception, self.period.close, [period])
 
+    def test0040check_origin(self):
+        'Test Move check_origin'
+        with Transaction().start(DB_NAME, USER, context=CONTEXT):
+            uom, = self.uom.search([('name', '=', 'Unit')])
+            template, = self.template.create([{
+                        'name': 'Test Move.check_origin',
+                        'type': 'goods',
+                        'list_price': Decimal(1),
+                        'cost_price': Decimal(0),
+                        'cost_price_method': 'fixed',
+                        'default_uom': uom.id,
+                        }])
+            product, = self.product.create([{
+                        'template': template.id,
+                        }])
+            storage, = self.location.search([('code', '=', 'STO')])
+            customer, = self.location.search([('code', '=', 'CUS')])
+            company, = self.company.search([
+                    ('rec_name', '=', 'Dunder Mifflin'),
+                    ])
 
-def doctest_dropdb(test):
-    '''
-    Remove sqlite memory database
-    '''
-    database = SQLiteDatabase().connect()
-    cursor = database.cursor(autocommit=True)
-    try:
-        database.drop(cursor, ':memory:')
-        cursor.commit()
-    finally:
-        cursor.close()
+            moves = self.move.create([{
+                        'product': product.id,
+                        'uom': uom.id,
+                        'quantity': 1,
+                        'from_location': storage.id,
+                        'to_location': customer.id,
+                        'company': company.id,
+                        'unit_price': Decimal(1),
+                        'currency': company.currency.id,
+                        }])
+
+            self.move.check_origin(moves, set())
+            self.move.check_origin(moves, {'supplier'})
+            self.assertRaises(UserWarning, self.move.check_origin, moves,
+                {'customer'})
 
 
 def suite():
@@ -538,7 +674,8 @@ def suite():
             'scenario_stock_average_cost_price.rst',
             setUp=doctest_dropdb, tearDown=doctest_dropdb, encoding='utf-8',
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_stock_inventory.rst',
+            setUp=doctest_dropdb, tearDown=doctest_dropdb, encoding='utf-8',
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     return suite
-
-if __name__ == '__main__':
-    unittest.TextTestRunner(verbosity=2).run(suite())
